#include <sys/param.h>
#include <sys/socket.h>
#include <sys/file.h>
#include <sys/time.h>
#include <sys/signal.h>

#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <errno.h>
#include <getopt.h>

/* private */
#include "sockopt.h"


/* defines */
#define DEFBLKSIZE (2048)
#define MAXBLKSIZE (32*1024)
#define DEFAULT_PORT	5001
#define ETH_HEAD_SIZE	14
#define IP_HEAD_SIZE	20
#define TCP_HEAD_SIZE	20
#define UDP_HEAD_SIZE	8
#define LLC_HEAD_SIZE	8
#define IPX_HEAD_SIZE	30
#define SPX_HEAD_SIZE	14
#define MTU_SIZE	    1500
#define TCP_DATA_SIZE	(MTU_SIZE-IP_HEAD_SIZE-TCP_HEAD_SIZE) /* 1460 */
#define UDP_DATA_SIZE	(MTU_SIZE-IP_HEAD_SIZE-UDP_HEAD_SIZE) /* 1472 */


/* variables */
const char *vdate="050213", verstr[30]="e-linux";
const char *progname = "netloop";

struct	sockaddr_in sock_in;
unsigned long starts, startms, stops, stopms, expms;
struct timeval ti; 
struct timezone tiz;

int	 wizsock = 0;
int  nblocks;
int  tcp=1;
int  udp=0;
int  randomb=0;
int  blksize=DEFBLKSIZE;
int  setbufsize=-1;
int  dots=0;
int  continuous=0;
int  experimental=0;
char port[30]="5001";
char buffer[MAXBLKSIZE];


/************************************************************************
 *	
 * Main.
 * 	
 ***********************************************************************/
 
static void usage(void)
{
	fprintf(stderr, "\n");
	fprintf(stderr, "usage: %s [options]\n\n", progname);
	fprintf(stderr, "Options:\n");
	fprintf(stderr, "-b nnn         socket buf size (default: %d == %s)\n", setbufsize, setbufsize==-1 ? "don't change" : "change");
	fprintf(stderr, "-c             display speed continuously\n");
	fprintf(stderr, "-d nnn         print dot every nnn blocks, 0 disables (default %d)\n", dots);
	fprintf(stderr, "-e             add experimental way of calculating throuput\n");
/*	fprintf(stderr, "-f FILE        send FILE instead of generated data\n"); */
	fprintf(stderr, "-h, --help     this help\n");
	fprintf(stderr, "-p xyz         use port #/name xyz instead of default %s\n", port);
	fprintf(stderr, "-r             send random data\n");
	fprintf(stderr, "-s nnn         block size (default %d bytes)\n", blksize);
	fprintf(stderr, "-t             use TCP\n");
	fprintf(stderr, "-u             use UDP\n");
	fprintf(stderr, "-V, --version  version\n");
	fprintf(stderr, "destination    host name or address\n");
	fprintf(stderr, "nblocks        number of blocks (1..9999)\n");
	fprintf(stderr, "\n");
	fprintf(stderr, "%s version: %s\n", progname, verstr);
	exit(1);
}

static void usage_small(void)
{
	fprintf(stderr, "type %s --help for help\n", progname);
}


int main (int argc, char *argv[])
{
	int i;
	int optchar;
	int bufsize;
	int family;
	int type;
	
	int sd, fd;
	int len, addr_len;
	struct sockaddr addr;
	int intr = 0;
	int r;
	time_t t;
	
	/* parse options */
    while ((optchar = getopt(argc, argv, "wtup:rs:b:d:Vhce")) != EOF)
	switch (optchar) {
		case '\0':
			break;
		case 'w':
			wizsock=1;
			break;
		case 't':
			tcp = 1;
			udp = 0;
			break;
		case 'u':
			udp = 1;
			tcp = 0;
			break;
		case 'r':
			srand(0 /* may be an option */);
			randomb = 1;
			break;
		case 's':
			blksize = abs(atoi(optarg));
			break;
		case 'b':
			setbufsize = abs(atoi(optarg));
			break;
		case 'c':
			continuous = 1;
			break;
		case 'd':
			dots = abs(atoi(optarg));
			break;
		case 'e':
			experimental = 1;
			break;
		case 'p': 
			strncpy(port, optarg, sizeof(port)-1);
			break;
		case 'V':
			printf("%s version: %s\n", progname, verstr);
			return 0;
			break;
		case 'h':
			usage();
	}
	
	argc -= optind;
	argv += optind;

/* correctness */
	if (tcp && udp) {
		printf("cannot use both TCP and UDP\n");
		usage_small();
		return -1;
	}
	if (!tcp && !udp) {
		printf("must use either TCP or UDP\n");
		usage_small();
		return -1;
	}
	
	if (continuous) dots=0;

	
	/* socket */
	family = wizsock ? AF_WIZNET : AF_INET;
	type = udp ? SOCK_DGRAM : SOCK_STREAM;
	bzero((char *)&sock_in, sizeof (sock_in));
	sock_in.sin_family = family;
	
	sd = socket(family, type, 0);
	if (sd < 0) {
		perror("socket");
		return -1;
	}
	
	setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, (void *)&i, sizeof(i));

    sock_in.sin_addr.s_addr = htonl(INADDR_ANY);
	sock_in.sin_port = htons(abs(atoi(port)));
	/* bind server port to socket */
	if (bind(sd, (struct sockaddr*)&sock_in, sizeof (sock_in)) < 0) {
		perror("bind");
		return -1;
	}
	
	/* listen */
	if (tcp) {
		listen(sd, 0);
	}

	printf("%s: %s waiting for data on port %s %s\n", progname, 
		family == AF_WIZNET ? "WIZNET":"INET", udp ? "UDP" : "TCP", port);
 	
 	if (udp || family == AF_WIZNET) {
		fd = sd;
	} else {
		addr_len = sizeof(addr);
		fd = accept(sd, &addr, &addr_len);
		if (fd < 0) {
			perror("cannot accept connection ");
			close(sd);
			return -1;
		}
	}
	
#if 0
	if (experimental && family == AF_WIZNET) {
		intr = 0;
		r = setsockopt(fd, SOL_PACKET, WIZ_SOPT_INTERRUPT, &intr, sizeof(intr));
		if (r < 0) printf("%s setsockopt: %s", progname, strerror(errno));
	}
#endif
	/* 0227:: wait until connect */
	if (experimental && family == AF_WIZNET) 
	{
		int opt_size, connected;

		intr = 0;
		opt_size = sizeof(int);

		r = setsockopt(fd, SOL_PACKET, WIZ_SOPT_INTERRUPT, &intr, sizeof(intr));
		if (r < 0) printf("%s setsockopt: %s", progname, strerror(errno));

		while ( 1 )
		{
			r = getsockopt(fd, SOL_PACKET, WIZ_SOPT_GETCONNECT, &connected, &opt_size);
			if (r < 0) 
			{
				printf("%s getsockopt: %s", progname, strerror(errno));
				return -1;
			}
			if (connected == 1) 
			{
				printf("%s: connection ok\n", progname);
				break;
			}
		}
	}
	/**-- 0227 */

	/* chungs */
	if ( blksize <= 0 )
		blksize = DEFBLKSIZE;
	/**********/

	while (1) {
		
		if (udp) {
			addr_len = sizeof(addr);
			len = recvfrom(fd, buffer, blksize, 0, &addr, &addr_len);
			if (len <= 0)
				break;
			sendto(fd, buffer, len, 0, &addr, addr_len);
			
		} else {
			if (experimental && family == AF_WIZNET) {
				struct s_wiznet_sockopt sreq;
				int opt_size, buf_len, connected;
				
				// get connection
				opt_size = sizeof(int);
				r = getsockopt(fd, SOL_PACKET, WIZ_SOPT_GETCONNECT, &connected, &opt_size);
				if (r < 0) {
					printf("%s getsockopt: %s", progname, strerror(errno));
					break;
				}
				if (connected == 0) {
					printf("%s: connection is terminated\n", progname);
					break;
				}
				
				// get received length.
				opt_size = sizeof(int);
				r = getsockopt(fd, SOL_PACKET, WIZ_SOPT_RXLEN, &buf_len, &opt_size);
				if (r < 0) {
					printf("%s getsockopt: %s", progname, strerror(errno));
					break;
				}
				if (buf_len == 0)
					continue;
				
				// read received data.
				sreq.len  = blksize;
				sreq.data = buffer;
				opt_size = sizeof(sreq);
				r = getsockopt(fd, SOL_PACKET, WIZ_SOPT_RXBUF, &sreq, &opt_size);
				if (r < 0) {
					printf("%s setsockopt: %s", progname, strerror(errno));
					break;
				}
				if (sreq.len == 0)
					continue;
				
				// wait to tx buffer
				t = time(NULL);
				do {
					// get received length.
					buf_len = 0;
					opt_size = sizeof(int);
					r = getsockopt(fd, SOL_PACKET, WIZ_SOPT_TXLEN, &buf_len, &opt_size);
					if (r < 0) {
						printf("%s getsockopt: %s", progname, strerror(errno));
						break;
					}
					
					if (buf_len < sreq.len && (time(NULL) - t) > 5) {
						printf("%s: wait to tx buffer(%d/%d) fail\n", progname, buf_len, sreq.len);
						goto errorExit;
					}
					
				} while (buf_len < sreq.len);
				
				// write received data.
				r = setsockopt(fd, SOL_PACKET, WIZ_SOPT_TXBUF, &sreq, sizeof(sreq));
				if (r < 0) {
					printf("%s setsockopt: %s", progname, strerror(errno));
					break;
				}
				
			} else {
				len = recv(fd, buffer, blksize, 0);
				if (len <= 0)
					break;
				write(fd, buffer, len);
			}
		}
    }

 errorExit:
	close(sd);
	
	return 0;
}
